__version__ = "3.4.7"

from .generators import rankcard
from .generators import trigger
from .generators import communism
from .generators import gay
from .generators import jail
from .generators import hitler
from .generators import spank
from .generators import aborted
from .generators import affect
from .generators import airpods
from .generators import america
from .generators import bed
from .generators import wanted
from .generators import jokeoverhead
from .generators import avatar
from .generators import welcomecard


from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw
from PIL import ImageColor
import asyncio
import aiohttp
from io import BytesIO
import discord
from typing import Union

async def getavatar(user: discord.Member) -> bytes:
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    disver = str(discord.__version__)
    if disver.startswith("1"):
        async with session.get(str(user.avatar_url)) as response:
            avatarbytes = await response.read()
        await session.close()
    elif disver.startswith("2"):
        async with session.get(str(user.display_avatar.url)) as response:
            avatarbytes = await response.read()
        await session.close()
    return avatarbytes

async def getbackground(background):
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    try:
        async with session.get(str(background)) as response:
            backgroundbytes = await response.read()
    except:
        background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
        async with session.get(str(background)) as response:
            backgroundbytes = await response.read()
    await session.close()
    return backgroundbytes

async def gettemplate(backtype, backdata=None):
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    if backdata == None:
        if backtype == "normal":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
        elif backtype == "transparent":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankopacityrank.png?v=1628032645740"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
        elif backtype == "font":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2FCalibri-Regular.ttf?v=1628033016557"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
    else:
        try:
            if backtype == "normal":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "transparent":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "font":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
        except:
            if backtype == "normal":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "transparent":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankopacityrank.png?v=1628032645740"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "font":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2FCalibri-Regular.ttf?v=1628033016557"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
    await session.close()
    return backgroundbytes

async def rankcard(user:discord.Member, username="Name", currentxp=1, lastxp=1, nextxp=1, level=1, rank=1, background=None, font=None, status=None, ranklevelsep="|", xpsep="/"):
        if type(rank) == int:
            rank = "#" + str(rank)
        big_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), 65, encoding="utf-8")
        medium_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), 50, encoding="utf-8")
        small_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), 40, encoding="utf-8")
        basecurrentxp = currentxp
        basenextxp = nextxp
        baselastxp = lastxp
        if level >= 1000000:
            if level >= 1000000:
                level = level / 1000000
                level = round(level, 1)
                level = str(level) + "M"
            else:
                level = level / 1000
                level = round(level, 1)
                level = str(level) + "K"
        else:
            level = str(level)
        if currentxp >= 1000:
            if currentxp >= 1000000:
                currentxp = currentxp / 1000000
                currentxp = round(currentxp, 1)
                currentxp = str(currentxp) + "M"
            else:
                currentxp = currentxp / 1000
                currentxp = round(currentxp, 1)
                currentxp = str(currentxp) + "K"
        if nextxp >= 1000:
            if nextxp >= 1000000:
                nextxp = nextxp / 1000000
                nextxp = round(nextxp, 1)
                nextxp = str(nextxp) + "M"
            else:
                nextxp = nextxp / 1000
                nextxp = round(nextxp, 1)
                nextxp = str(nextxp) + "K"
        if lastxp >= 1000:
            if lastxp >= 1000000:
                lastxp = lastxp / 1000000
                lastxp = round(lastxp, 1)
                lastxp = str(lastxp) + "M"
            else:
                lastxp = lastxp / 1000
                lastxp = round(lastxp, 1)
                lastxp = str(lastxp) + "K"
        with Image.open(BytesIO(await getavatar(user))) as im:
                    im = im.resize((183, 183))
                    rgbavatar = im.convert("RGB")
                    if background == None:
                        with Image.open(BytesIO(await gettemplate("normal"))).convert('RGB') as background:
                            draw = ImageDraw.Draw(background)
                            draw.ellipse((35, 57, 206, 226), fill=0)
                    else:
                        with Image.open(BytesIO(await getbackground(background))) as background:
                            background = background.resize((934, 282))
                            draw = ImageDraw.Draw(background)
                            opacity = Image.open(BytesIO(await gettemplate("transparent"))).convert("RGBA")
                            newbackground = background.convert("RGBA")
                            width = (background.width - newbackground.width) // 2
                            height = (background.height - newbackground.height) // 2
                            background.paste(opacity, (width, height), opacity)
                            draw.ellipse((35, 57, 206, 226), fill=0)
                    with Image.new("L", im.size, 0) as mask:
                        mask_draw = ImageDraw.Draw(mask)
                        mask_draw.ellipse([(20, 20), im.size], fill=255)
                        background.paste(rgbavatar, (20, 40), mask=mask)
                    draw.ellipse((160, 170, 208, 218), fill=0)
                    try:
                        if status == None:
                            if user.status == discord.Status.online:
                                draw.ellipse((165, 175, 204, 214), fill=(67,181,129))
                            elif user.status == discord.Status.offline:
                                draw.ellipse((165, 175, 204, 214), fill=(116, 127, 141))
                            elif user.status == discord.Status.dnd:
                                draw.ellipse((165, 175, 204, 214), fill=(240,71,71))
                            elif user.status == discord.Status.idle:
                                draw.ellipse((165, 175, 204, 214), fill=(250,166,26))
                        else:
                            if status == "online":
                                draw.ellipse((165, 175, 204, 214), fill=(67,181,129))
                            elif status == "offline":
                                draw.ellipse((165, 175, 204, 214), fill=(116, 127, 141))
                            elif status == "dnd":
                                draw.ellipse((165, 175, 204, 214), fill=(240,71,71))
                            elif status == "idle":
                                draw.ellipse((165, 175, 204, 214), fill=(250,166,26))
                    except:
                        draw.ellipse((165, 175, 204, 214), fill=(114,137,218))
                    if len(username) > 12:
                        changelen = len(username) - 12
                        if changelen > -0:
                            newlen = 50 - round(1*changelen)
                        else:
                            newlen = 50
                        medium_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), newlen, encoding="utf-8")
                    draw.text((270, 130), "{}".format(username), (255, 255, 255), anchor="lt", font=medium_font)
                    medium_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), 50, encoding="utf-8")
                    draw.text((875, 135), "{} {} {} XP".format(currentxp, xpsep, nextxp), (255, 255, 255),  anchor="rt", font=small_font)
                    draw.text((875, 55), "Rank {} {} Level {}".format(rank, ranklevelsep, level), (255, 255, 255), anchor="rt", font=medium_font)
                    if isinstance(user, discord.Member) or isinstance(user, discord.User):
                        colour = user.colour.to_rgb()
                        if colour == (0, 0, 0):
                            colour = (37, 115, 189)
                    else:
                        colour = (37, 115, 189)
                    x=257
                    y=183
                    w=597
                    h=35
                    progress = (int(basecurrentxp) - int(baselastxp)) / (int(basenextxp)- int(baselastxp))
                    '''PROGRESS'''
                    if(progress<=0):        
                        progress = 0.01    
                    if(progress>1):        
                        progress=.99
                    w = w*progress    
                    draw.ellipse((x+w, y, x+h+w, y+h),fill=colour)
                    draw.ellipse((x, y, x+h, y+h),fill=colour)
                    draw.rectangle((x+(h/2), y, x+w+(h/2), y+h),fill=colour)
                    final_buffer = BytesIO()
                    background.save(final_buffer, "png")

        final_buffer.seek(0)
        return final_buffer


from .rankcard import rankcard
from .trigger import trigger
from .communism import communism
from .gay import gay
from .jail import jail
from .hitler import hitler
from .spank import spank
from .aborted import aborted
from .affect import affect
from .airpods import airpods
from .america import america
from .bed import bed
from .wanted import wanted
from .jokeoverhead import jokeoverhead
from .avatar import avatar
from .welcomecard import welcomecard


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = """[![Discord](https://discord.com/api/guilds/872291125547921459/embed.png)](https://discord.gg/mPU3HybBs9)
[![PyPi](https://img.shields.io/pypi/v/canvacord.svg)](https://pypi.org/project/canvacord)
[![Python](https://img.shields.io/pypi/pyversions/dislash.py.svg)](https://pypi.python.org/pypi/canvacord)

A copy of [canvacord](https://www.npmjs.com/package/canvacord) made in python!



# Table of contents
1. [Installation](#installation)
2. [Examples](#examples)
3. [Creating Images](#creating-images)
4. [Links](#links)
5. [Downloads](#downloads)

# Installation

Run any of these commands in terminal:

Mac / Linux
```
pip install canvacord
```

Windows
```
python -m pip install canvacord
```

# Examples
💡 This library requires **[discord.py](https://github.com/Rapptz/discord.py)**.


## Creating-Images

```python
import discord
from discord.ext import commands
import canvacord

client = commands.Bot(command_prefix="!")

@client.command()
async def rankcard(ctx):
    user = ctx.author
    username = ctx.author.name + "#" + ctx.author.discriminator
    currentxp = 1
    lastxp = 0
    nextxp = 2
    current_level = 1
    current_rank = 1
    background = None
    image = await canvacord.rankcard(user=user, username=username, currentxp=currentxp, lastxp=lastxp, nextxp=nextxp, level=current_level, rank=current_rank, background=background, ranklevelsep="|", xpsep="/")
    file = discord.File(filename="rankcard.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def welcomecard(ctx):
    image = await canvacord.welcomecard(user=ctx.author, background=None, avatarcolor="white", topcolor="white", bottomcolor="white", backgroundcolor="black", font=None, toptext="Welcome {user_name}!", bottomtext="Enjoy your stay in {server}!")
    file = discord.File(filename="welcomecard.png", fp=image)
    await ctx.send(file=file)
    
@client.command()
async def triggered(ctx):
    user = ctx.author
    image = await canvacord.trigger(user)
    file = discord.File(filename="triggered.gif", fp=image)
    await ctx.send(file=file)

@client.command()
async def communism(ctx):
    user = ctx.author
    image = await canvacord.communism(user)
    file = discord.File(filename="communism.gif", fp=image)
    await ctx.send(file=file)

@client.command()
async def jail(ctx):
    user = ctx.author
    image = await canvacord.jail(user)
    file = discord.File(filename="jail.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def gay(ctx):
    user = ctx.author
    image = await canvacord.gay(user)
    file = discord.File(filename="gay.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def hitler(ctx):
    user = ctx.author
    image = await canvacord.hitler(user)
    file = discord.File(filename="hitler.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def aborted(ctx):
    user = ctx.author
    image = await canvacord.aborted(user)
    file = discord.File(filename="aborted.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def affect(ctx):
    user = ctx.author
    image = await canvacord.affect(user)
    file = discord.File(filename="affect.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def airpods(ctx):
    user = ctx.author
    image = await canvacord.airpods(user)
    file = discord.File(filename="airpods.gif", fp=image)
    await ctx.send(file=file)

@client.command()
async def america(ctx):
    user = ctx.author
    image = await canvacord.america(user)
    file = discord.File(filename="america.gif", fp=image)
    await ctx.send(file=file)

@client.command()
async def wanted(ctx):
    user = ctx.author
    image = await canvacord.wanted(user)
    file = discord.File(filename="wanted.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def joke(ctx):
    user = ctx.author
    image = await canvacord.jokeoverhead(user)
    file = discord.File(filename="jokeoverhead.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def spank(ctx):
    user1 = ctx.author
    user2 = ctx.author
    image = await canvacord.spank(user1, user2)
    file = discord.File(filename="spank.png", fp=image)
    await ctx.send(file=file)

@client.command()
async def bed(ctx):
    user1 = ctx.author
    user2 = ctx.author
    image = await canvacord.bed(user1, user2)
    file = discord.File(filename="bed.png", fp=image)
    await ctx.send(file=file)
    
client.run("BOT_TOKEN")
```


# Links
[![Discord](https://discord.com/api/guilds/872291125547921459/embed.png)](https://discord.gg/mPU3HybBs9)
[![PyPi](https://img.shields.io/pypi/v/canvacord.svg)](https://pypi.org/project/canvacord)


# Downloads


[![Downloads](https://pepy.tech/badge/canvacord)](https://pepy.tech/project/canvacord)
[![Downloads](https://pepy.tech/badge/canvacord/month)](https://pepy.tech/project/canvacord)
![Downloads](https://pepy.tech/badge/canvacord/week)
"""

with open('src/canvacord/__init__.py', 'r') as f:
    version = [line.split('=')[1].strip(" '\"") for line in f.read().splitlines() if line.startswith('__version__')][0]


setup(
    name='canvacord',
    version=version,
    description='A Python Version of Canvacord',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    url='https://github.com/BlazenBoi/canvacord.py/issues',
    author='Blazen',
    author_email='contact@fireballbot.com',
    keywords='canvacord, rankcard, image manipulation, meme, discord, discordpy, discord-py',
    packages=find_packages(include=['canvacord']),
    python_requires='>=3.6',
    install_requires=[
    "setuptools>=42",
    "wheel",
    "pillow",
    "discord",
    "disnake",
    "datetime",
    "requests",
    "colour"
    ],
    project_urls={
        'Discord Server': 'https://discord.com/invite/mPU3HybBs9',
        'Bug Tracker': 'https://github.com/BlazenBoi/canvacord.py/issues',
        'Source': 'https://github.com/BlazenBoi/canvacord',
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
      ]
)
#© 2021 GitHub, Inc.


from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw
from PIL import ImageColor
import asyncio
import aiohttp
from io import BytesIO
import discord
from typing import Union
from colour import Color

async def getavatar(user: discord.Member) -> bytes:
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    disver = str(discord.__version__)
    if disver.startswith("1"):
        async with session.get(str(user.avatar_url)) as response:
            avatarbytes = await response.read()
        await session.close()
    elif disver.startswith("2"):
        async with session.get(str(user.display_avatar.url)) as response:
            avatarbytes = await response.read()
        await session.close()
    return avatarbytes

async def getbackground(background):
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    try:
        async with session.get(str(background)) as response:
            backgroundbytes = await response.read()
    except:
        background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
        async with session.get(str(background)) as response:
            backgroundbytes = await response.read()
    await session.close()
    return backgroundbytes

async def gettemplate(backtype, backdata=None):
    session = aiohttp.ClientSession(loop=asyncio.get_event_loop())
    if backdata == None:
        if backtype == "normal":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
        elif backtype == "transparent":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankopacityrank.png?v=1628032645740"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
        elif backtype == "font":
            background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2FCalibri-Regular.ttf?v=1628033016557"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
        elif backtype == "welcome":
            background = "https://cdn.glitch.global/dff50ce1-3805-4fdb-a7a5-8cabd5e53756/thumbnails%2Fimage.png?1700612054798"
            async with session.get(str(background)) as response:
                backgroundbytes = await response.read()
    else:
        try:
            if backtype == "normal":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "transparent":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "font":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "welcome":
                async with session.get(str(backdata)) as response:
                    backgroundbytes = await response.read()
        except:
            if backtype == "normal":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankrank.png?v=1628032652421"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "transparent":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2Fblankopacityrank.png?v=1628032645740"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "font":
                background = "https://cdn.glitch.com/dff50ce1-3805-4fdb-a7a5-8cabd5e53756%2FCalibri-Regular.ttf?v=1628033016557"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
            elif backtype == "welcome":
                background = "https://cdn.glitch.global/dff50ce1-3805-4fdb-a7a5-8cabd5e53756/thumbnails%2Fimage.png?1700612054798"
                async with session.get(str(background)) as response:
                    backgroundbytes = await response.read()
    await session.close()
    return backgroundbytes

async def welcomecard(user:discord.Member, background=None, avatarcolor="white", topcolor="white", bottomcolor="white", backgroundcolor="black", font=None, toptext="Welcome {user_name}!", bottomtext="Enjoy your stay in {server}!"):
        toptext = toptext.replace("{user_name}", user.name)
        bottomtext = bottomtext.replace("{user_name}", user.name)
        toptext = toptext.replace("{server}", user.guild.name)
        bottomtext = bottomtext.replace("{server}", user.guild.name)
        ac = Color(avatarcolor)
        tc = Color(topcolor)
        bc = Color(bottomcolor)
        bgc = Color(backgroundcolor)
        medium_font = ImageFont.FreeTypeFont(BytesIO(await gettemplate("font", font)), 50, encoding="utf-8")
        with Image.open(BytesIO(await getavatar(user))) as im:
            im = im.resize((256, 256))
            rgbavatar = im.convert("RGB")
            if background == None:
                background = Image.new("RGB", (1024, 500), bgc.hex_l)
                draw = ImageDraw.Draw(background)
            else:
                background = Image.open(BytesIO(await getbackground(background)))
                background = background.resize((1024, 500))
                draw = ImageDraw.Draw(background)
                background = background.convert("RGBA")
                draw = ImageDraw.Draw(background)
            draw.ellipse([(384, 19), (640, 275)], fill=ac.hex_l)
            with Image.new("L", im.size, 0) as mask:
                mask_draw = ImageDraw.Draw(mask)
                mask_draw.ellipse([(20, 20), im.size], fill=255)
                background.paste(rgbavatar, (374, 10), mask=mask)
            msg1 = toptext
            w = medium_font.getlength(msg1)
            draw.text(((1024-w)/2,300), msg1, fill=tc.hex_l, font=medium_font)
            msg2 = bottomtext
            w = medium_font.getlength(msg2)
            draw.text(((1024-w)/2,375), msg2, fill=bc.hex_l, font=medium_font)
            final_buffer = BytesIO()
            background.save(final_buffer, "png")
            final_buffer.seek(0)
            return final_buffer